package build.gist.gist_flutter

import androidx.annotation.NonNull
import android.content.Context
import android.app.Application
import android.os.Handler
import android.os.Looper
import build.gist.data.model.Message
import build.gist.presentation.GistListener

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import build.gist.presentation.GistSdk
import io.flutter.plugin.common.EventChannel

private const val METHODS_CHANNEL = "gist_flutter"
private const val EVENTS_CHANNEL = "gist_flutter_events"

class GistFlutterPlugin: FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler, GistListener {
  private lateinit var methodChannel : MethodChannel
  private lateinit var eventChannel: EventChannel
  private lateinit var context: Context
  private var eventSink: EventChannel.EventSink? = null
  private val handler = Handler(Looper.getMainLooper())

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    context = flutterPluginBinding.applicationContext
    methodChannel = MethodChannel(flutterPluginBinding.binaryMessenger, METHODS_CHANNEL)
    methodChannel.setMethodCallHandler(this)

    eventChannel = EventChannel(flutterPluginBinding.binaryMessenger, EVENTS_CHANNEL)
    eventChannel.setStreamHandler(this)
    GistSdk.addListener(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "setup" -> {
        val application : Application = context.applicationContext as Application
        GistSdk.getInstance().init(application, call.arguments.toString())
      }
      "setUserToken" -> GistSdk.getInstance().setUserToken(call.arguments.toString())
      "clearUserToken" -> GistSdk.getInstance().clearUserToken()
      "setCurrentRoute" -> GistSdk.getInstance().setCurrentRoute(call.arguments.toString())
      "subscribeToTopic" -> GistSdk.getInstance().subscribeToTopic(call.arguments.toString())
      "unsubscribeFromTopic" -> GistSdk.getInstance().unsubscribeFromTopic(call.arguments.toString())
      "clearTopics" -> GistSdk.getInstance().clearTopics()
      else -> result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    methodChannel.setMethodCallHandler(null)
  }

  override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
    events?.let { eventSink -> this.eventSink = eventSink }
  }

  override fun onCancel(arguments: Any?) {
    eventSink = null
  }

  // Events

  override fun onMessageShown(message: Message) {
    handler.post {
      val mappedMessage = mapMessage("onMessageShown", message)
      eventSink?.let { eventSink -> eventSink.success(mappedMessage) }
    }
  }

  override fun onMessageDismissed(message: Message) {
    handler.post {
      val mappedMessage = mapMessage("onMessageDismissed", message)
      eventSink?.let { eventSink -> eventSink.success(mappedMessage) }
    }
  }

  override fun onError(message: Message) {
    handler.post {
      val mappedMessage = mapMessage("onError", message)
      eventSink?.let { eventSink -> eventSink.success(mappedMessage) }
    }
  }

  override fun onAction(message: Message, currentRoute: String, action: String) {
    handler.post {
      val mappedMessage = mapMessage("onAction", message)
      mappedMessage["currentRoute"] = currentRoute
      mappedMessage["action"] = action
      eventSink?.let { eventSink -> eventSink.success(mappedMessage) }
    }
  }

  override fun embedMessage(message: Message, elementId: String) {
    // Not implemented in Flutter
  }

  fun mapMessage(eventName: String, message: Message): HashMap<String, Any?> {
    val params = HashMap<String, Any?>()
    params["eventName"] = eventName
    params["messageId"] = message.messageId
    params["instanceId"] = message.instanceId
    params["queueId"] = message.queueId
    return params
  }
}
